#include "utils.hpp"
#include <libscerror.hpp>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <string>

using namespace sc;

term::term(int fd) {
    if (isatty(fd)) {
        throw_if_min1(ioctl(fd, TIOCGWINSZ, &_ws));
    }
    switch (fd) {
        case STDOUT_FILENO: _out = &std::cout; break;
        case STDERR_FILENO: _out = &std::cerr; break;
        default: break;
    }
}

int term::rows() const {
    return _ws.ws_row;
}

int term::cols() const {
    return _ws.ws_col;
}

void term::progress(double cur, double total) const {
    // use unicode to make nice bar
    const char* bar[8] = {
        u8"\u258F", u8"\u258E", u8"\u258D", u8"\u258C",
        u8"\u258B", u8"\u258A", u8"\u2589", u8"\u2588"
    };
    int barwidth = cols() - 5;
    int maxsteps = barwidth * 8;
    int steps = round(cur * maxsteps / total);
    int perc = round(100 * cur / total);
    *_out << '\r' << std::setw(3) << perc << "% ";
    for (int i = 0; i < steps / 8; ++i) {
        *_out << bar[7];
    }
    if (steps % 8) {
        *_out << bar[steps % 8 - 1];
    }
}

void term::hide_cursor() const {
    *_out << "\x1b[?25l";
}

void term::show_cursor() const {
    *_out << "\x1b[?25h";
}

cursor_hider::cursor_hider(int fd) : _t {fd} {
    _t.hide_cursor();
}

cursor_hider::~cursor_hider() {
    _t.show_cursor();
}
